-------------------------------------------------------------------------------
-- Localized Lua globals
-------------------------------------------------------------------------------
local _G = getfenv(0)

local string = _G.string

local pairs = _G.pairs
local wipe = _G.table.wipe 

-------------------------------------------------------------------------------
-- AddOn namespace
-------------------------------------------------------------------------------
local MODNAME = "Syllabus"
local Syllabus = LibStub("AceAddon-3.0"):NewAddon(MODNAME, "AceEvent-3.0", "AceConsole-3.0")
_G[MODNAME] = Syllabus

local LDB = LibStub("LibDataBroker-1.1")
local QTip = LibStub("LibQTip-1.0")

local dev = false
--[===[@debug@
dev = true
--@end-debug@]===]
local L = LibStub("AceLocale-3.0"):GetLocale(MODNAME, "enUS", true, dev)

local DataObj
local tooltip

local version = GetAddOnMetadata(MODNAME, "Version")
version = string.gsub(version, "@project.version@", " - Development")

-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------
local MY_CLASS, MY_CLASS_UPPER = UnitClass("player")
local MY_NAME = UnitName("player")
local MY_RACE = UnitRace("player")

local COLOR_TABLE = CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS
local CLASS_COLORS = {}

local MY_PROFESSIONS = {}

for k, v in pairs(COLOR_TABLE) do
	CLASS_COLORS[k] = string.format("%2x%2x%2x", v.r * 255, v.g * 255, v.b * 255)
end

-------------------------------------------------------------------------------
-- Variables
-------------------------------------------------------------------------------
local update_queued = false
local in_combat = false

local db

-------------------------------------------------------------------------------
-- Functions
-------------------------------------------------------------------------------
local function UpdateSkills()
	if in_combat then
		update_queued = true
		return
	end
	wipe(MY_PROFESSIONS)

	for i = 1, GetNumSkillLines() do
		local name, is_header, _, rank, _, _, max_rank,	_, _, _, _, _, _ = GetSkillLineInfo(i);

		if not is_header and not string.find(name, LANGUAGE) and max_rank > 1 then
			MY_PROFESSIONS[name] = MY_PROFESSIONS[name] or {}
			MY_PROFESSIONS[name].rank = rank
			MY_PROFESSIONS[name].max_rank = max_rank
			db.char.known[name] = true
		end
	end
end


local function ShowTooltip(anchor)
	local level = UnitLevel("player")
	local line, column

	tooltip = QTip:Acquire(MODNAME.."Tooltip", 3, "LEFT", "LEFT", "LEFT")
	tooltip:SetAutoHideDelay(0.25, anchor)
	tooltip:SmartAnchorTo(anchor)
	tooltip:Clear()

	line, column = tooltip:AddHeader()

	tooltip:SetCell(line, column,
			string.format("%s - %s %d %s |cff%s%s|r", MY_NAME, LEVEL, level, MY_RACE, CLASS_COLORS[MY_CLASS_UPPER], MY_CLASS), "CENTER", 3)
	tooltip:AddLine()
	tooltip:Show()
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------
function Syllabus:PLAYER_REGEN_DISABLED()
	in_combat = true
end

function Syllabus:PLAYER_REGEN_ENABLED()
	in_combat = false

	if update_queued then
		update_queued = false
		UpdateSkills()
	end
end

function Syllabus:TRADE_SKILL_SHOW()
	if IsTradeSkillLinked() then	-- We don't want to save the information if this isn't our tradeskill list.
		return
	end
	DataObj:SetIcon()

	for i = 1, GetNumTradeSkills() do
		local name, type, _, _, _ = GetTradeSkillInfo(i)

		if name and type ~= "header" then
--			Syllabus:Print("Adding "..name)
			db.char.known[name] = true
		end
	end
end

do
	local current_skill = {}	-- Re-usable table for skill insertions.

	local type_filters = {
		["available"] = 0,
		["unavailable"] = 0,
		["used"] = 0,
	}
	local DataMiner = CreateFrame("GameTooltip", "SyllabusDataMiner", UIParent, "GameTooltipTemplate")
	DataMiner:SetOwner(UIParent, "ANCHOR_NONE")

	function Syllabus:TRAINER_SHOW()
		GameTooltip_SetDefaultAnchor(DataMiner, UIParent)
		DataMiner:SetUnit("npc")

		local trainer_name = _G["SyllabusDataMinerTextLeft1"]:GetText()
		local trainer_role = _G["SyllabusDataMinerTextLeft2"]:GetText()
		DataMiner:Hide()

		local trainer_x, trainer_y = GetPlayerMapPosition("player")
		local trainer_location = GetRealZoneText()

		self:Print(string.format("Trainer is: %s <%s>, in %s at %.2f, %.2f.", trainer_name, trainer_role, trainer_location, trainer_x * 100, trainer_y * 100))

		-- Save the user's preferences before opening everything.
		for k, v in pairs(type_filters) do
			type_filters[k] = GetTrainerServiceTypeFilter(k) or 0
			SetTrainerServiceTypeFilter(k, 1)
		end

		for i = 1, GetNumTrainerServices() do
			local _ , _, category = GetTrainerServiceInfo(i)

			if not category then
				break
			end

			if category == "header" then
				ExpandTrainerSkillLine(i)
			end
		end

		if IsTradeskillTrainer() then
			self:Print("Tradeskill!")

			if not MY_PROFESSIONS[GetTrainerServiceSkillLine(2)] then
				self:Print("Not saving - not our profession.")
				return
			end
		end
		local trainer_db = db.global.trainer

		trainer_db[trainer_role] = trainer_db[trainer_role] or {}
		trainer_db[trainer_role].members = trainer_db[trainer_role].members or {}
		trainer_db[trainer_role].members[trainer_name] = trainer_db[trainer_role].members[trainer_name] or {}

		local trainer_info = trainer_db[trainer_role].members[trainer_name]
		trainer_info.x = trainer_x
		trainer_info.y = trainer_y
		trainer_info.location = trainer_location

		trainer_db[trainer_role].teaches = trainer_db[trainer_role].teaches or {}

		for idx = 1, GetNumTrainerServices() do
			local svc_name, svc_subtxt, svc_type, _ = GetTrainerServiceInfo(idx)

			if svc_name and svc_type and svc_type ~= "header" then
				if svc_subtxt ~= "" then
					svc_name = svc_name .. " (" .. svc_subtxt .. ")"
				end
				local level_req	= GetTrainerServiceLevelReq(idx)
				local skill_req, rank_req, _ = GetTrainerServiceSkillReq(idx)

				if rank_req == 0 then
					skill_req = nil
					rank_req = nil
				end
				local num_reqs = GetTrainerServiceNumAbilityReq(idx)
				local ability_reqs, meets_reqs

				if num_reqs > 0 then
					ability_reqs, meets_reqs = GetTrainerServiceAbilityReq(idx, 1)
				end

				local item_link = GetTrainerServiceItemLink(idx)

				current_skill.item_link = item_link and item_link:match("item:(%d+)") or nil
				current_skill.icon = GetTrainerServiceIcon(idx)
				current_skill.cost = GetTrainerServiceCost(idx)
				current_skill.level_req = ((level_req > 0) and level_req) or nil
				current_skill.description = ((not current_skill.item_link) and GetTrainerServiceDescription(idx)) or nil
				current_skill.skill_req = skill_req
				current_skill.rank_req = rank_req
				current_skill.requires = ability_reqs

				local svc_header = GetTrainerServiceSkillLine(idx)

				trainer_db[trainer_role].teaches[svc_header] = trainer_db[trainer_role].teaches[svc_header] or {}
				trainer_db[trainer_role].teaches[svc_header][svc_name] = trainer_db[trainer_role].teaches[svc_header][svc_name] or {}

				for key, val in pairs(current_skill) do
					trainer_db[trainer_role].teaches[svc_header][svc_name][key] = val
				end

				if svc_type == "used" and svc_name ~= svc_header then
					db.char.known[svc_name] = true
				end
			end
		end

		-- Restore the previous values for the user's type filters.
		for k, v in pairs(type_filters) do
			SetTrainerServiceTypeFilter(k, type_filters[k])
		end
		DataObj:SetIcon()
	end
end	-- do

-------------------------------------------------------------------------------
-- Initialization functions
-------------------------------------------------------------------------------
function Syllabus:OnInitialize()
	DataObj = LDB:NewDataObject(MODNAME, {
		type	= "data source",
		label	= MODNAME,
		text	= "0",
		icon	= "Interface\\Icons\\INV_Misc_Book_01",
		SetIcon	= function(self)
				  local num = math.random(15)
				  self.icon = "Interface\\Icons\\INV_Misc_Book_"..((num < 10) and "0" or "")..num
			  end,
		OnEnter	= function(display, motion)
				  ShowTooltip(display)
			  end,
		OnLeave	= function(display, motion)
			  end,
	})
	local defaults = {
		global = {
			trainer = {},
			-- Configuration data
			show = {
				cost = true,
				known = false,
				not_trainable = false,
				overall_cost = true,
				requirements = true,
				skill_count = true,
				trainable = true,
				trainable_future = true,
				trainer_names = true,
			},
			level_range = 2,
			skill_range = 5,
		},
		char = {
			known = {},
			hidden = {},
		},
	}
	db = LibStub("AceDB-3.0"):New(MODNAME.."DB", defaults)
--	self:SetupOptions()
	self.OnInitialize = nil
end

function Syllabus:OnEnable()
	UpdateSkills()
	self:RegisterEvent("PLAYER_LEVEL_UP", UpdateSkills)
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("SKILL_LINES_CHANGED", UpdateSkills)
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("TRAINER_SHOW")
end

function Syllabus:OnDisable()
	self:UnregisterEvent("PLAYER_LEVEL_UP")
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("SKILL_LINES_CHANGED")
	self:UnregisterEvent("TRADE_SKILL_SHOW")
	self:UnregisterEvent("TRAINER_SHOW")
end
